﻿using System;
using System.IO;
using System.Text;
using System.Windows;
using System.Net;
using System.Threading;
using System.Web;
using System.Drawing;
using System.Windows.Media.Imaging;
using System.Configuration;

namespace Micro.Listener
{
    /// <summary>
    /// 同步处理
    /// 调用示例：
    /// 一、启动服务：new Micro.Listener.ListenerSync(8181).Start();
    /// 二、资源调用：
    /// 1、HTML：webBrower.Source = new Uri("http://localhost:8080/pack://application:,,,/Micro.UI;component/template.html");
    /// 2、CSS/JS：<link href="http://localhost:8080/pack://application:,,,/Micro.UI;component/res/style.css" rel="stylesheet" />
    /// 3、图片：<img src="http://localhost:8080/pack://application:,,,/Micro.UI;component/res/logo.png"/>  
    /// </summary>
    public class ListenerSync
    {
        public int listenport { set; get; }
        public ListenerSync()
        {
            this.listenport = 8080;
        }
        public ListenerSync(int _listenport)
        {
            this.listenport = _listenport;
        }
        HttpListener listener;
        /// <summary>
        /// 构造简HTTP服务器
        /// </summary>
        public void Start(bool IsAsync = false)
        {
            if (listener == null)
            {
                listener = new HttpListener();
                listener.AuthenticationSchemes = AuthenticationSchemes.Anonymous;
                listener.Prefixes.Add(String.Format("http://localhost:{0}/", this.listenport));
                listener.Start();

                new Thread(new ThreadStart(delegate
                {
                    while (true)
                    {
                        HttpListenerContext context = listener.GetContext();
                        string rawUrl = context.Request.RawUrl.TrimStart('/');
                        string ContentType, FileType = ListenerUtils.GetFileType(rawUrl, out ContentType);
                        //检索是否为资源文件
                        if (ListenerUtils.IsUriResource(rawUrl))
                        {
                            //开始同步检索传入的请求
                            using (StreamWriter writer = new StreamWriter(context.Response.OutputStream, Encoding.UTF8))
                            {
                                Uri uri = new Uri(rawUrl, UriKind.Absolute);
                                Stream stream = Application.GetResourceStream(uri).Stream;
                                if (FileType.Equals("text"))
                                {
                                    StreamReader reader = new StreamReader(stream);
                                    writer.WriteLine(reader.ReadToEnd());
                                }
                                if (FileType.Equals("image"))
                                {
                                    byte[] bytes = new byte[stream.Length];
                                    stream.Read(bytes, 0, bytes.Length);
                                    // 设置当前流的位置为流的开始
                                    stream.Seek(0, SeekOrigin.Begin);
                                    // 输出内容
                                    context.Response.OutputStream.Write(bytes, 0, bytes.Length);
                                    context.Response.ContentType = ContentType;
                                }
                            }
                        }
                    }
                })).Start();
            }
        }
    }
}
